# Usage Example

from enstools.feature.pipeline import FeaturePipeline
from enstools.feature.identification.storm import StormIdentification
from enstools.feature.tracking.overlap_tracking import OverlapTracking
from enstools.feature.identification._proto_gen import storm_pb2
from os.path import expanduser
from enstools.feature.identification.storm.util import plot_timestep, plot_track
from enstools.feature.tracking.overlap_double_threshold_tracking import OverlapDoubleThresholdTracking

import os
from datetime import timedelta

from enstools.feature.util.graph import DataGraph

# set the pb_reference to the compiled pb2.py file (see proto_gen directory)
pipeline = FeaturePipeline(storm_pb2, processing_mode='2d')

# change this to an identification strategy that actually does something: existing one or implement your own
i_strat = StormIdentification(some_parameter='foo') # set the Identification strategy

t_strat = OverlapTracking(field_name='storm_areas', min_duration=timedelta(hours=12)) # set the tracking strategy
# t_strat = OverlapDoubleThresholdTracking(
#    inner_thresh_name='storm_areas',
#    outer_thresh_name='storm_areas',
#    tracking_method='inner_first',
#    min_duration=timedelta(hours=12)
#    )

pipeline.set_identification_strategy(i_strat)
pipeline.set_tracking_strategy(t_strat) # or None as argument if no tracking

path = '/project/meteo/w2w/workshops/EMS_2022/feature/data/storm/zeynep.nc'

plot_dir = os.path.expanduser('~/plots/')
if not os.path.exists(plot_dir):
    os.makedirs(plot_dir)

desc_dir = os.path.expanduser('~/feature_desc/')
if not os.path.exists(desc_dir):
    os.makedirs(desc_dir)

pipeline.set_data_path(path)

# execute pipeline
pipeline.execute()

od = pipeline.get_object_desc()

reanalysis_set = od.sets[0]

g = DataGraph(reanalysis_set, t_strat)
g.generate_tracks(apply_filter=True)

# subset = get_subset_by_description(pipeline.get_data(), reanalysis_set, '2d')

for ts in reanalysis_set.timesteps:
    ds_t = pipeline.get_data().sel(time=ts.valid_time)

    plot_timestep(ds_t, plot_dir + str(ts.valid_time)[:13] + '.png')

for track_id, track in enumerate(reanalysis_set.tracks):
    plot_track(track, plot_dir + 'track' + str(track_id) + '.png', pipeline.get_data().storm_areas)


out_json_path = desc_dir + 'desc.json'
out_ds_path = desc_dir + 'desc.nc'

pipeline.save_result(description_type='json', description_path=out_json_path, dataset_path=out_ds_path)


